@@ -45,7 +45,7 @@
 #include "isisd/isis_sr.h"
 #include "isisd/isis_ldp_sync.h"
 
-struct zclient *zclient;
+struct zclient *isis_zclient;
 static struct zclient *zclient_sync;
 
 /* Router-id update message from zebra. */
@@ -171,6 +171,9 @@ static int isis_zebra_add_nexthops(struct isis *isis, struct list *nexthops,
 		if (count >= MULTIPATH_NUM)
 			break;
 		api_nh = &zapi_nexthops[count];
+
+		zapi_nexthop_init(api_nh);
+
 		if (fabricd)
 			SET_FLAG(api_nh->flags, ZAPI_NEXTHOP_FLAG_ONLINK);
 		api_nh->vrf_id = isis->vrf_id;
@@ -254,7 +257,7 @@ void isis_zebra_route_add_route(struct isis *isis, struct prefix *prefix,
 	struct zapi_route api;
 	int count = 0;
 
-	if (zclient->sock < 0)
+	if (isis_zclient->sock < 0)
 		return;
 
 	/* Uninstall the route if it doesn't have any valid nexthop. */
@@ -263,7 +266,7 @@ void isis_zebra_route_add_route(struct isis *isis, struct prefix *prefix,
 		return;
 	}
 
-	memset(&api, 0, sizeof(api));
+	zapi_route_init(&api);
 	api.vrf_id = isis->vrf_id;
 	api.type = PROTO_TYPE;
 	api.safi = SAFI_UNICAST;
@@ -295,7 +298,7 @@ void isis_zebra_route_add_route(struct isis *isis, struct prefix *prefix,
 		return;
 	api.nexthop_num = count;
 
-	zclient_route_send(ZEBRA_ROUTE_ADD, zclient, &api);
+	zclient_route_send(ZEBRA_ROUTE_ADD, isis_zclient, &api);
 }
 
 void isis_zebra_route_del_route(struct isis *isis,
@@ -305,7 +308,7 @@ void isis_zebra_route_del_route(struct isis *isis,
 {
 	struct zapi_route api;
 
-	if (zclient->sock < 0)
+	if (isis_zclient->sock < 0)
 		return;
 
 	if (!CHECK_FLAG(route_info->flag, ISIS_ROUTE_FLAG_ZEBRA_SYNCED))
@@ -321,7 +324,7 @@ void isis_zebra_route_del_route(struct isis *isis,
 		SET_FLAG(api.message, ZAPI_MESSAGE_SRCPFX);
 	}
 
-	zclient_route_send(ZEBRA_ROUTE_DELETE, zclient, &api);
+	zclient_route_send(ZEBRA_ROUTE_DELETE, isis_zclient, &api);
 }
 
 /**
@@ -388,7 +391,7 @@ void isis_zebra_prefix_sid_install(struct isis_area *area,
 	}
 
 	/* Send message to zebra. */
-	(void)zebra_send_mpls_labels(zclient, ZEBRA_MPLS_LABELS_REPLACE, &zl);
+	(void)zebra_send_mpls_labels(isis_zclient, ZEBRA_MPLS_LABELS_REPLACE, &zl);
 }
 
 /**
@@ -415,7 +418,7 @@ void isis_zebra_prefix_sid_uninstall(struct isis_area *area,
 	zl.local_label = psid->label;
 
 	/* Send message to zebra. */
-	(void)zebra_send_mpls_labels(zclient, ZEBRA_MPLS_LABELS_DELETE, &zl);
+	(void)zebra_send_mpls_labels(isis_zclient, ZEBRA_MPLS_LABELS_DELETE, &zl);
 }
 
 /**
@@ -471,7 +474,7 @@ void isis_zebra_send_adjacency_sid(int cmd, const struct sr_adjacency *sra)
 		}
 	}
 
-	(void)zebra_send_mpls_labels(zclient, cmd, &zl);
+	(void)zebra_send_mpls_labels(isis_zclient, cmd, &zl);
 }
 
 static int isis_zebra_read(ZAPI_CALLBACK_ARGS)
@@ -523,9 +526,9 @@ void isis_zebra_redistribute_set(afi_t afi, int type, vrf_id_t vrf_id,
 {
 	if (type == DEFAULT_ROUTE)
 		zclient_redistribute_default(ZEBRA_REDISTRIBUTE_DEFAULT_ADD,
-					     zclient, afi, vrf_id);
+					     isis_zclient, afi, vrf_id);
 	else
-		zclient_redistribute(ZEBRA_REDISTRIBUTE_ADD, zclient, afi, type,
+		zclient_redistribute(ZEBRA_REDISTRIBUTE_ADD, isis_zclient, afi, type,
 				     tableid, vrf_id);
 }
 
@@ -534,9 +537,9 @@ void isis_zebra_redistribute_unset(afi_t afi, int type, vrf_id_t vrf_id,
 {
 	if (type == DEFAULT_ROUTE)
 		zclient_redistribute_default(ZEBRA_REDISTRIBUTE_DEFAULT_DELETE,
-					     zclient, afi, vrf_id);
+					     isis_zclient, afi, vrf_id);
 	else
-		zclient_redistribute(ZEBRA_REDISTRIBUTE_DELETE, zclient, afi,
+		zclient_redistribute(ZEBRA_REDISTRIBUTE_DELETE, isis_zclient, afi,
 				     type, tableid, vrf_id);
 }
 
@@ -549,7 +552,7 @@ int isis_zebra_rlfa_register(struct isis_spftree *spftree, struct rlfa *rlfa)
 	struct zapi_rlfa_request zr = {};
 	int ret;
 
-	if (!zclient)
+	if (!isis_zclient)
 		return 0;
 
 	zr.igp.vrf_id = area->isis->vrf_id;
@@ -565,7 +568,7 @@ int isis_zebra_rlfa_register(struct isis_spftree *spftree, struct rlfa *rlfa)
 	zlog_debug("ISIS-LFA: registering RLFA %pFX@%pI4 with LDP",
 		   &rlfa->prefix, &rlfa->pq_address);
 
-	ret = zclient_send_opaque_unicast(zclient, LDP_RLFA_REGISTER,
+	ret = zclient_send_opaque_unicast(isis_zclient, LDP_RLFA_REGISTER,
 					  ZEBRA_ROUTE_LDP, 0, 0,
 					  (const uint8_t *)&zr, sizeof(zr));
 	if (ret == ZCLIENT_SEND_FAILURE) {
@@ -585,8 +588,8 @@ void isis_zebra_rlfa_unregister_all(struct isis_spftree *spftree)
 	struct zapi_rlfa_igp igp = {};
 	int ret;
 
-	if (!zclient || spftree->type != SPF_TYPE_FORWARD
-	    || CHECK_FLAG(spftree->flags, F_SPFTREE_NO_ADJACENCIES))
+	if (!isis_zclient || spftree->type != SPF_TYPE_FORWARD ||
+	    CHECK_FLAG(spftree->flags, F_SPFTREE_NO_ADJACENCIES))
 		return;
 
 	if (IS_DEBUG_LFA)
@@ -599,7 +602,7 @@ void isis_zebra_rlfa_unregister_all(struct isis_spftree *spftree)
 	igp.isis.spf.level = spftree->level;
 	igp.isis.spf.run_id = spftree->runcount;
 
-	ret = zclient_send_opaque_unicast(zclient, LDP_RLFA_UNREGISTER_ALL,
+	ret = zclient_send_opaque_unicast(isis_zclient, LDP_RLFA_UNREGISTER_ALL,
 					  ZEBRA_ROUTE_LDP, 0, 0,
 					  (const uint8_t *)&igp, sizeof(igp));
 	if (ret == ZCLIENT_SEND_FAILURE)
@@ -669,6 +672,7 @@ void isis_zebra_request_srv6_sid_endx(struct isis_adjacency *adj)
 
 	ctx.behavior = ZEBRA_SEG6_LOCAL_ACTION_END_X;
 	ctx.nh6 = nexthop;
+	ctx.ifindex = circuit->interface->ifindex;
 	ret = isis_zebra_request_srv6_sid(&ctx, &sid_value,
 					  area->srv6db.config.srv6_locator_name);
 	if (!ret) {
@@ -774,27 +778,27 @@ int isis_zebra_label_manager_connect(void)
 
 void isis_zebra_vrf_register(struct isis *isis)
 {
-	if (!zclient || zclient->sock < 0 || !isis)
+	if (!isis_zclient || isis_zclient->sock < 0 || !isis)
 		return;
 
 	if (isis->vrf_id != VRF_UNKNOWN) {
 		if (IS_DEBUG_EVENTS)
 			zlog_debug("%s: Register VRF %s id %u", __func__,
 				   isis->name, isis->vrf_id);
-		zclient_send_reg_requests(zclient, isis->vrf_id);
+		zclient_send_reg_requests(isis_zclient, isis->vrf_id);
 	}
 }
 
 void isis_zebra_vrf_deregister(struct isis *isis)
 {
-	if (!zclient || zclient->sock < 0 || !isis)
+	if (!isis_zclient || isis_zclient->sock < 0 || !isis)
 		return;
 
 	if (isis->vrf_id != VRF_UNKNOWN) {
 		if (IS_DEBUG_EVENTS)
 			zlog_debug("%s: Deregister VRF %s id %u", __func__,
 				   isis->name, isis->vrf_id);
-		zclient_send_dereg_requests(zclient, isis->vrf_id);
+		zclient_send_dereg_requests(isis_zclient, isis->vrf_id);
 	}
 }
 
@@ -820,9 +824,9 @@ int isis_zebra_ls_register(bool up)
 	int rc;
 
 	if (up)
-		rc = ls_register(zclient, true);
+		rc = ls_register(isis_zclient, true);
 	else
-		rc = ls_unregister(zclient, true);
+		rc = ls_unregister(isis_zclient, true);
 
 	return rc;
 }
@@ -888,73 +892,6 @@ static int isis_zebra_client_close_notify(ZAPI_CALLBACK_ARGS)
 	return ret;
 }
 
-/**
- * Send SRv6 SID to ZEBRA for installation or deletion.
- *
- * @param cmd		ZEBRA_ROUTE_ADD or ZEBRA_ROUTE_DELETE
- * @param sid		SRv6 SID to install or delete
- * @param prefixlen	Prefix length
- * @param oif		Outgoing interface
- * @param action	SID action
- * @param context	SID context
- */
-static void isis_zebra_send_localsid(int cmd, const struct in6_addr *sid,
-				     uint16_t prefixlen, ifindex_t oif,
-				     enum seg6local_action_t action,
-				     const struct seg6local_context *context)
-{
-	struct prefix_ipv6 p = {};
-	struct zapi_route api = {};
-	struct zapi_nexthop *znh;
-
-	if (cmd != ZEBRA_ROUTE_ADD && cmd != ZEBRA_ROUTE_DELETE) {
-		flog_warn(EC_LIB_DEVELOPMENT, "%s: wrong ZEBRA command",
-			  __func__);
-		return;
-	}
-
-	if (prefixlen > IPV6_MAX_BITLEN) {
-		flog_warn(EC_LIB_DEVELOPMENT, "%s: wrong prefixlen %u",
-			  __func__, prefixlen);
-		return;
-	}
-
-	sr_debug("  |- %s SRv6 SID %pI6 behavior %s",
-		 cmd == ZEBRA_ROUTE_ADD ? "Add" : "Delete", sid,
-		 seg6local_action2str(action));
-
-	p.family = AF_INET6;
-	p.prefixlen = prefixlen;
-	p.prefix = *sid;
-
-	api.vrf_id = VRF_DEFAULT;
-	api.type = PROTO_TYPE;
-	api.instance = 0;
-	api.safi = SAFI_UNICAST;
-	memcpy(&api.prefix, &p, sizeof(p));
-
-	if (cmd == ZEBRA_ROUTE_DELETE)
-		return (void)zclient_route_send(ZEBRA_ROUTE_DELETE, zclient,
-						&api);
-
-	SET_FLAG(api.flags, ZEBRA_FLAG_ALLOW_RECURSION);
-	SET_FLAG(api.message, ZAPI_MESSAGE_NEXTHOP);
-
-	znh = &api.nexthops[0];
-
-	memset(znh, 0, sizeof(*znh));
-
-	znh->type = NEXTHOP_TYPE_IFINDEX;
-	znh->ifindex = oif;
-	SET_FLAG(znh->flags, ZAPI_NEXTHOP_FLAG_SEG6LOCAL);
-	znh->seg6local_action = action;
-	memcpy(&znh->seg6local_ctx, context, sizeof(struct seg6local_context));
-
-	api.nexthop_num = 1;
-
-	zclient_route_send(ZEBRA_ROUTE_ADD, zclient, &api);
-}
-
 /**
  * Install SRv6 SID in the forwarding plane through Zebra.
  *
@@ -980,6 +917,11 @@ void isis_zebra_srv6_sid_install(struct isis_area *area,
 		action = ZEBRA_SEG6_LOCAL_ACTION_END;
 		prefixlen = IPV6_MAX_BITLEN;
 		break;
+	case SRV6_ENDPOINT_BEHAVIOR_END_PSP:
+		action = ZEBRA_SEG6_LOCAL_ACTION_END;
+		prefixlen = IPV6_MAX_BITLEN;
+		SET_SRV6_FLV_OP(ctx.flv.flv_ops, ZEBRA_SEG6_LOCAL_FLV_OP_PSP);
+		break;
 	case SRV6_ENDPOINT_BEHAVIOR_END_NEXT_CSID:
 		action = ZEBRA_SEG6_LOCAL_ACTION_END;
 		prefixlen = sid->locator->block_bits_length +
@@ -989,6 +931,14 @@ void isis_zebra_srv6_sid_install(struct isis_area *area,
 		ctx.flv.lcblock_len = sid->locator->block_bits_length;
 		ctx.flv.lcnode_func_len = sid->locator->node_bits_length;
 		break;
+	case SRV6_ENDPOINT_BEHAVIOR_END_NEXT_CSID_PSP:
+		action = ZEBRA_SEG6_LOCAL_ACTION_END;
+		prefixlen = sid->locator->block_bits_length + sid->locator->node_bits_length;
+		SET_SRV6_FLV_OP(ctx.flv.flv_ops, ZEBRA_SEG6_LOCAL_FLV_OP_NEXT_CSID);
+		SET_SRV6_FLV_OP(ctx.flv.flv_ops, ZEBRA_SEG6_LOCAL_FLV_OP_PSP);
+		ctx.flv.lcblock_len = sid->locator->block_bits_length;
+		ctx.flv.lcnode_func_len = sid->locator->node_bits_length;
+		break;
 	case SRV6_ENDPOINT_BEHAVIOR_END_X:
 	case SRV6_ENDPOINT_BEHAVIOR_END_X_NEXT_CSID:
 	case SRV6_ENDPOINT_BEHAVIOR_RESERVED:
@@ -999,6 +949,16 @@ void isis_zebra_srv6_sid_install(struct isis_area *area,
 	case SRV6_ENDPOINT_BEHAVIOR_END_DT4_USID:
 	case SRV6_ENDPOINT_BEHAVIOR_END_DT46_USID:
 	case SRV6_ENDPOINT_BEHAVIOR_OPAQUE:
+	case SRV6_ENDPOINT_BEHAVIOR_END_PSP_USD:
+	case SRV6_ENDPOINT_BEHAVIOR_END_NEXT_CSID_PSP_USD:
+	case SRV6_ENDPOINT_BEHAVIOR_END_X_PSP:
+	case SRV6_ENDPOINT_BEHAVIOR_END_X_PSP_USD:
+	case SRV6_ENDPOINT_BEHAVIOR_END_X_NEXT_CSID_PSP:
+	case SRV6_ENDPOINT_BEHAVIOR_END_X_NEXT_CSID_PSP_USD:
+	case SRV6_ENDPOINT_BEHAVIOR_END_B6_ENCAPS:
+	case SRV6_ENDPOINT_BEHAVIOR_END_B6_ENCAPS_RED:
+	case SRV6_ENDPOINT_BEHAVIOR_END_B6_ENCAPS_NEXT_CSID:
+	case SRV6_ENDPOINT_BEHAVIOR_END_B6_ENCAPS_RED_NEXT_CSID:
 	default:
 		zlog_err(
 			"ISIS-SRv6 (%s): unsupported SRv6 endpoint behavior %u",
@@ -1016,8 +976,8 @@ void isis_zebra_srv6_sid_install(struct isis_area *area,
 	}
 
 	/* Send the SID to zebra */
-	isis_zebra_send_localsid(ZEBRA_ROUTE_ADD, &sid->sid, prefixlen,
-				 ifp->ifindex, action, &ctx);
+	zclient_send_localsid(isis_zclient, ZEBRA_ROUTE_ADD, &sid->sid, prefixlen, ifp->ifindex,
+			      action, &ctx);
 }
 
 /**
@@ -1040,9 +1000,11 @@ void isis_zebra_srv6_sid_uninstall(struct isis_area *area,
 
 	switch (sid->behavior) {
 	case SRV6_ENDPOINT_BEHAVIOR_END:
+	case SRV6_ENDPOINT_BEHAVIOR_END_PSP:
 		prefixlen = IPV6_MAX_BITLEN;
 		break;
 	case SRV6_ENDPOINT_BEHAVIOR_END_NEXT_CSID:
+	case SRV6_ENDPOINT_BEHAVIOR_END_NEXT_CSID_PSP:
 		prefixlen = sid->locator->block_bits_length +
 			    sid->locator->node_bits_length;
 		break;
@@ -1056,6 +1018,16 @@ void isis_zebra_srv6_sid_uninstall(struct isis_area *area,
 	case SRV6_ENDPOINT_BEHAVIOR_END_DT4_USID:
 	case SRV6_ENDPOINT_BEHAVIOR_END_DT46_USID:
 	case SRV6_ENDPOINT_BEHAVIOR_OPAQUE:
+	case SRV6_ENDPOINT_BEHAVIOR_END_PSP_USD:
+	case SRV6_ENDPOINT_BEHAVIOR_END_NEXT_CSID_PSP_USD:
+	case SRV6_ENDPOINT_BEHAVIOR_END_X_PSP:
+	case SRV6_ENDPOINT_BEHAVIOR_END_X_PSP_USD:
+	case SRV6_ENDPOINT_BEHAVIOR_END_X_NEXT_CSID_PSP:
+	case SRV6_ENDPOINT_BEHAVIOR_END_X_NEXT_CSID_PSP_USD:
+	case SRV6_ENDPOINT_BEHAVIOR_END_B6_ENCAPS:
+	case SRV6_ENDPOINT_BEHAVIOR_END_B6_ENCAPS_RED:
+	case SRV6_ENDPOINT_BEHAVIOR_END_B6_ENCAPS_NEXT_CSID:
+	case SRV6_ENDPOINT_BEHAVIOR_END_B6_ENCAPS_RED_NEXT_CSID:
 	default:
 		zlog_err(
 			"ISIS-SRv6 (%s): unsupported SRv6 endpoint behavior %u",
@@ -1072,8 +1044,8 @@ void isis_zebra_srv6_sid_uninstall(struct isis_area *area,
 	}
 
 	/* Send delete request to zebra */
-	isis_zebra_send_localsid(ZEBRA_ROUTE_DELETE, &sid->sid, prefixlen,
-				 ifp->ifindex, action, NULL);
+	zclient_send_localsid(isis_zclient, ZEBRA_ROUTE_DELETE, &sid->sid, prefixlen, ifp->ifindex,
+			      action, NULL);
 }
 
 void isis_zebra_srv6_adj_sid_install(struct srv6_adjacency *sra)
@@ -1121,6 +1093,18 @@ void isis_zebra_srv6_adj_sid_install(struct srv6_adjacency *sra)
 	case SRV6_ENDPOINT_BEHAVIOR_END_DT4_USID:
 	case SRV6_ENDPOINT_BEHAVIOR_END_DT46_USID:
 	case SRV6_ENDPOINT_BEHAVIOR_OPAQUE:
+	case SRV6_ENDPOINT_BEHAVIOR_END_PSP:
+	case SRV6_ENDPOINT_BEHAVIOR_END_PSP_USD:
+	case SRV6_ENDPOINT_BEHAVIOR_END_NEXT_CSID_PSP:
+	case SRV6_ENDPOINT_BEHAVIOR_END_NEXT_CSID_PSP_USD:
+	case SRV6_ENDPOINT_BEHAVIOR_END_X_PSP:
+	case SRV6_ENDPOINT_BEHAVIOR_END_X_PSP_USD:
+	case SRV6_ENDPOINT_BEHAVIOR_END_X_NEXT_CSID_PSP:
+	case SRV6_ENDPOINT_BEHAVIOR_END_X_NEXT_CSID_PSP_USD:
+	case SRV6_ENDPOINT_BEHAVIOR_END_B6_ENCAPS:
+	case SRV6_ENDPOINT_BEHAVIOR_END_B6_ENCAPS_RED:
+	case SRV6_ENDPOINT_BEHAVIOR_END_B6_ENCAPS_NEXT_CSID:
+	case SRV6_ENDPOINT_BEHAVIOR_END_B6_ENCAPS_RED_NEXT_CSID:
 	default:
 		zlog_err(
 			"ISIS-SRv6 (%s): unsupported SRv6 endpoint behavior %u",
@@ -1130,8 +1114,8 @@ void isis_zebra_srv6_adj_sid_install(struct srv6_adjacency *sra)
 
 	ifp = sra->adj->circuit->interface;
 
-	isis_zebra_send_localsid(ZEBRA_ROUTE_ADD, &sra->sid, prefixlen,
-				 ifp->ifindex, action, &ctx);
+	zclient_send_localsid(isis_zclient, ZEBRA_ROUTE_ADD, &sra->sid, prefixlen, ifp->ifindex,
+			      action, &ctx);
 }
 
 void isis_zebra_srv6_adj_sid_uninstall(struct srv6_adjacency *sra)
@@ -1167,6 +1151,18 @@ void isis_zebra_srv6_adj_sid_uninstall(struct srv6_adjacency *sra)
 	case SRV6_ENDPOINT_BEHAVIOR_END_DT4_USID:
 	case SRV6_ENDPOINT_BEHAVIOR_END_DT46_USID:
 	case SRV6_ENDPOINT_BEHAVIOR_OPAQUE:
+	case SRV6_ENDPOINT_BEHAVIOR_END_PSP:
+	case SRV6_ENDPOINT_BEHAVIOR_END_PSP_USD:
+	case SRV6_ENDPOINT_BEHAVIOR_END_NEXT_CSID_PSP:
+	case SRV6_ENDPOINT_BEHAVIOR_END_NEXT_CSID_PSP_USD:
+	case SRV6_ENDPOINT_BEHAVIOR_END_X_PSP:
+	case SRV6_ENDPOINT_BEHAVIOR_END_X_PSP_USD:
+	case SRV6_ENDPOINT_BEHAVIOR_END_X_NEXT_CSID_PSP:
+	case SRV6_ENDPOINT_BEHAVIOR_END_X_NEXT_CSID_PSP_USD:
+	case SRV6_ENDPOINT_BEHAVIOR_END_B6_ENCAPS:
+	case SRV6_ENDPOINT_BEHAVIOR_END_B6_ENCAPS_RED:
+	case SRV6_ENDPOINT_BEHAVIOR_END_B6_ENCAPS_NEXT_CSID:
+	case SRV6_ENDPOINT_BEHAVIOR_END_B6_ENCAPS_RED_NEXT_CSID:
 	default:
 		zlog_err(
 			"ISIS-SRv6 (%s): unsupported SRv6 endpoint behavior %u",
@@ -1179,8 +1175,8 @@ void isis_zebra_srv6_adj_sid_uninstall(struct srv6_adjacency *sra)
 	sr_debug("ISIS-SRv6 (%s): delete End.X SID %pI6", area->area_tag,
 		 &sra->sid);
 
-	isis_zebra_send_localsid(ZEBRA_ROUTE_DELETE, &sra->sid, prefixlen,
-				 ifp->ifindex, action, NULL);
+	zclient_send_localsid(isis_zclient, ZEBRA_ROUTE_DELETE, &sra->sid, prefixlen, ifp->ifindex,
+			      action, NULL);
 }
 
 /**
@@ -1340,7 +1336,7 @@ static int isis_zebra_process_srv6_locator_delete(ZAPI_CALLBACK_ARGS)
  */
 int isis_zebra_srv6_manager_get_locator_chunk(const char *name)
 {
-	return srv6_manager_get_locator_chunk(zclient, name);
+	return srv6_manager_get_locator_chunk(isis_zclient, name);
 }
 
 
@@ -1353,7 +1349,7 @@ int isis_zebra_srv6_manager_get_locator_chunk(const char *name)
  */
 int isis_zebra_srv6_manager_release_locator_chunk(const char *name)
 {
-	return srv6_manager_release_locator_chunk(zclient, name);
+	return srv6_manager_release_locator_chunk(isis_zclient, name);
 }
 
 /**
@@ -1371,7 +1367,7 @@ int isis_zebra_srv6_manager_get_locator(const char *name)
 	 * Send the Get Locator request to the SRv6 Manager and return the
 	 * result
 	 */
-	return srv6_manager_get_locator(zclient, name);
+	return srv6_manager_get_locator(isis_zclient, name);
 }
 
 /**
@@ -1402,7 +1398,7 @@ bool isis_zebra_request_srv6_sid(const struct srv6_sid_ctx *ctx,
 	 * Send the Get SRv6 SID request to the SRv6 Manager and check the
 	 * result
 	 */
-	ret = srv6_manager_get_sid(zclient, ctx, sid_value, locator_name, NULL);
+	ret = srv6_manager_get_sid(isis_zclient, ctx, sid_value, locator_name, NULL, false);
 	if (ret < 0) {
 		zlog_warn("%s: error getting SRv6 SID!", __func__);
 		return false;
@@ -1418,19 +1414,20 @@ bool isis_zebra_request_srv6_sid(const struct srv6_sid_ctx *ctx,
  * to use the SID.
  *
  * @param ctx Context to be associated with the SID to be released
+ * @param locator_name Parent locator of the SID
  */
-void isis_zebra_release_srv6_sid(const struct srv6_sid_ctx *ctx)
+void isis_zebra_release_srv6_sid(const struct srv6_sid_ctx *ctx, const char *locator_name)
 {
 	int ret;
 
-	if (!ctx)
+	if (!ctx || !locator_name)
 		return;
 
 	/*
 	 * Send the Release SRv6 SID request to the SRv6 Manager and check the
 	 * result
 	 */
-	ret = srv6_manager_release_sid(zclient, ctx);
+	ret = srv6_manager_release_sid(isis_zclient, ctx, locator_name, false);
 	if (ret < 0) {
 		zlog_warn("%s: error releasing SRv6 SID!", __func__);
 		return;
@@ -1505,11 +1502,8 @@ static int isis_zebra_srv6_sid_notify(ZAPI_CALLBACK_ARGS)
 				}
 
 				/* Allocate new SRv6 End SID */
-				behavior =
-					(CHECK_FLAG(locator->flags,
-						    SRV6_LOCATOR_USID))
-						? SRV6_ENDPOINT_BEHAVIOR_END_NEXT_CSID
-						: SRV6_ENDPOINT_BEHAVIOR_END;
+				behavior = SRV6_BEHAVIOR_CODEPOINT(SRV6_ENDPOINT_BEHAVIOR_END,
+								   locator->flags);
 				sid = isis_srv6_sid_alloc(area,
 							  area->srv6db
 								  .srv6_locator,
@@ -1600,16 +1594,16 @@ static zclient_handler *const isis_handlers[] = {
 	[ZEBRA_SRV6_SID_NOTIFY] = isis_zebra_srv6_sid_notify,
 };
 
-void isis_zebra_init(struct event_loop *master, int instance)
+void isis_zebra_init(struct event_loop *mst, int instance)
 {
 	/* Initialize asynchronous zclient. */
-	zclient = zclient_new(master, &zclient_options_default, isis_handlers,
+	isis_zclient = zclient_new(mst, &zclient_options_default, isis_handlers,
 			      array_size(isis_handlers));
-	zclient_init(zclient, PROTO_TYPE, 0, &isisd_privs);
-	zclient->zebra_connected = isis_zebra_connected;
+	zclient_init(isis_zclient, PROTO_TYPE, 0, &isisd_privs);
+	isis_zclient->zebra_connected = isis_zebra_connected;
 
 	/* Initialize special zclient for synchronous message exchanges. */
-	zclient_sync = zclient_new(master, &zclient_options_sync, NULL, 0);
+	zclient_sync = zclient_new(mst, &zclient_options_sync, NULL, 0);
 	zclient_sync->sock = -1;
 	zclient_sync->redist_default = ZEBRA_ROUTE_ISIS;
 	zclient_sync->instance = instance;
@@ -1623,11 +1617,11 @@ void isis_zebra_init(struct event_loop *master, int instance)
 
 void isis_zebra_stop(void)
 {
-	zclient_unregister_opaque(zclient, LDP_RLFA_LABELS);
-	zclient_unregister_opaque(zclient, LDP_IGP_SYNC_IF_STATE_UPDATE);
-	zclient_unregister_opaque(zclient, LDP_IGP_SYNC_ANNOUNCE_UPDATE);
+	zclient_unregister_opaque(isis_zclient, LDP_RLFA_LABELS);
+	zclient_unregister_opaque(isis_zclient, LDP_IGP_SYNC_IF_STATE_UPDATE);
+	zclient_unregister_opaque(isis_zclient, LDP_IGP_SYNC_ANNOUNCE_UPDATE);
 	zclient_stop(zclient_sync);
 	zclient_free(zclient_sync);
-	zclient_stop(zclient);
-	zclient_free(zclient);
+	zclient_stop(isis_zclient);
+	zclient_free(isis_zclient);
 }
