@@ -28,6 +28,7 @@
 #include "stream.h"
 #include "network.h"
 #include "libfrr.h"
+#include "zlog_live.h"
 
 static void		 lde_shutdown(void);
 static void lde_dispatch_imsg(struct event *thread);
@@ -116,6 +117,8 @@ static struct frr_signal_t lde_signals[] =
 void
 lde(void)
 {
+	static struct zlog_live_cfg child_log;
+
 #ifdef HAVE_SETPROCTITLE
 	setproctitle("label decision engine");
 #endif
@@ -123,6 +126,8 @@ lde(void)
 	log_procname = log_procnames[PROC_LDE_ENGINE];
 
 	master = frr_init();
+	zlog_live_open_fd(&child_log, LOG_DEBUG, LDPD_FD_LOG);
+
 	/* no frr_config_fork() here, allow frr_pthread to create threads */
 	frr_is_after_fork = true;
 
@@ -171,8 +176,7 @@ lde_init(struct ldpd_init *init)
 	zclient_sync_init();
 }
 
-static void
-lde_shutdown(void)
+static FRR_NORETURN void lde_shutdown(void)
 {
 	/* close pipes */
 	if (iev_ldpe) {
@@ -395,12 +399,55 @@ static void lde_dispatch_imsg(struct event *thread)
 		imsg_event_add(iev);
 	else {
 		/* this pipe is dead, so remove the event handlers and exit */
-		EVENT_OFF(iev->ev_read);
-		EVENT_OFF(iev->ev_write);
+		event_cancel(&iev->ev_read);
+		event_cancel(&iev->ev_write);
 		lde_shutdown();
 	}
 }
 
+/* ARGSUSED */
+static void lde_send_all_klabel(struct iface *iface)
+{
+	struct fec *fec;
+	struct fec_node *fn;
+	struct fec_nh *fnh;
+	struct lde_map *me;
+	struct lde_nbr *ln;
+
+	RB_FOREACH (ln, nbr_tree, &lde_nbrs) {
+		RB_FOREACH (fec, fec_tree, &ln->recv_map) {
+			switch (fec->type) {
+			case FEC_TYPE_IPV4:
+				break;
+			case FEC_TYPE_IPV6:
+				break;
+			case FEC_TYPE_PWID:
+			default:
+				continue;
+			}
+
+			fn = (struct fec_node *)fec_find(&ft, fec);
+			if (fn == NULL) {
+				/* shouldn't happen */
+				continue;
+			}
+
+			LIST_FOREACH (fnh, &fn->nexthops, entry) {
+				if (fnh->ifindex != iface->ifindex)
+					continue;
+
+				if (lde_address_find(ln, fnh->af, &fnh->nexthop) == NULL)
+					continue;
+
+				me = (struct lde_map *)fec;
+				fnh->remote_label = me->map.label;
+				lde_send_change_klabel(fn, fnh);
+				break;
+			}
+		}
+	}
+}
+
 /* ARGSUSED */
 static void lde_dispatch_parent(struct event *thread)
 {
@@ -450,8 +497,11 @@ static void lde_dispatch_parent(struct event *thread)
 				if_update_info(iface, kif);
 
 				/* if up see if any labels need to be updated */
-				if (kif->operative)
+				if (kif->operative) {
 					lde_route_update(iface, AF_UNSPEC);
+					lde_send_all_klabel(iface);
+				}
+
 				break;
 			}
 
@@ -533,8 +583,8 @@ static void lde_dispatch_parent(struct event *thread)
 			    sizeof(struct ldpd_init))
 				fatalx("INIT imsg with wrong len");
 
-			memcpy(&init, imsg.data, sizeof(init));
-			lde_init(&init);
+			memcpy(&ldp_init, imsg.data, sizeof(ldp_init));
+			lde_init(&ldp_init);
 			break;
 		case IMSG_AGENTX_ENABLED:
 			ldp_agentx_enabled();
@@ -673,8 +723,8 @@ static void lde_dispatch_parent(struct event *thread)
 		imsg_event_add(iev);
 	else {
 		/* this pipe is dead, so remove the event handlers and exit */
-		EVENT_OFF(iev->ev_read);
-		EVENT_OFF(iev->ev_write);
+		event_cancel(&iev->ev_read);
+		event_cancel(&iev->ev_write);
 		lde_shutdown();
 	}
 }
@@ -2135,12 +2185,8 @@ static void zclient_sync_retry(struct event *thread)
  */
 static void zclient_sync_init(void)
 {
-	struct zclient_options options = zclient_options_default;
-
-	options.synchronous = true;
-
 	/* Initialize special zclient for synchronous message exchanges. */
-	zclient_sync = zclient_new(master, &options, NULL, 0);
+	zclient_sync = zclient_new(master, &zclient_options_sync, NULL, 0);
 	zclient_sync->sock = -1;
 	zclient_sync->redist_default = ZEBRA_ROUTE_LDP;
 	zclient_sync->session_id = 1; /* Distinguish from main session */
@@ -2444,6 +2490,8 @@ void lde_route_update_release(struct iface *iface, int af)
 				continue;
 
 			SET_FLAG(fnh->flags, F_FEC_NH_NO_LDP);
+			lde_send_delete_klabel(fn, fnh);
+
 			RB_FOREACH(ln, nbr_tree, &lde_nbrs)
 				lde_send_labelwithdraw(ln, fn, NULL, NULL);
 			lde_free_label(fn->local_label);
