@@ -40,7 +40,7 @@ int main(int argc, char **argv)
 
 	vty_init(master, true);
 	lib_cmd_init();
-	nb_init(master, NULL, 0, false);
+	nb_init(master, NULL, 0, false, false);
 
 	vty_stdio(vty_do_exit);
 
