@@ -427,25 +427,11 @@ static struct lyd_node *get_dnode_config(const std::string &path)
 	return dnode;
 }
 
-static int get_oper_data_cb(const struct lysc_node *snode,
-			    struct yang_translator *translator,
-			    struct yang_data *data, void *arg)
-{
-	struct lyd_node *dnode = static_cast<struct lyd_node *>(arg);
-	int ret = yang_dnode_edit(dnode, data->xpath, data->value);
-	yang_data_free(data);
-
-	return (ret == 0) ? NB_OK : NB_ERR;
-}
-
 static struct lyd_node *get_dnode_state(const std::string &path)
 {
-	struct lyd_node *dnode = yang_dnode_new(ly_native_ctx, false);
-	if (nb_oper_data_iterate(path.c_str(), NULL, 0, get_oper_data_cb, dnode)
-	    != NB_OK) {
-		yang_dnode_free(dnode);
-		return NULL;
-	}
+	struct lyd_node *dnode = NULL;
+
+	(void)nb_oper_iterate_legacy(path.c_str(), NULL, 0, NULL, NULL, &dnode);
 
 	return dnode;
 }
@@ -1025,12 +1011,11 @@ grpc::Status HandleUnaryExecute(
 	grpc_debug("%s: entered", __func__);
 
 	struct nb_node *nb_node;
-	struct list *input_list;
-	struct list *output_list;
-	struct listnode *node;
-	struct yang_data *data;
+	struct lyd_node *input_tree, *output_tree, *child;
 	const char *xpath;
 	char errmsg[BUFSIZ] = {0};
+	char path[XPATH_MAXLEN];
+	LY_ERR err;
 
 	// Request: string path = 1;
 	xpath = tag->request.path().c_str();
@@ -1046,40 +1031,66 @@ grpc::Status HandleUnaryExecute(
 		return grpc::Status(grpc::StatusCode::INVALID_ARGUMENT,
 				    "Unknown data path");
 
-	input_list = yang_data_list_new();
-	output_list = yang_data_list_new();
+	// Create input data tree.
+	err = lyd_new_path2(NULL, ly_native_ctx, xpath, NULL, 0,
+			    (LYD_ANYDATA_VALUETYPE)0, 0, NULL, &input_tree);
+	if (err != LY_SUCCESS) {
+		return grpc::Status(grpc::StatusCode::INVALID_ARGUMENT,
+				    "Invalid data path");
+	}
 
 	// Read input parameters.
 	auto input = tag->request.input();
 	for (const frr::PathValue &pv : input) {
 		// Request: repeated PathValue input = 2;
-		data = yang_data_new(pv.path().c_str(), pv.value().c_str());
-		listnode_add(input_list, data);
+		err = lyd_new_path(input_tree, ly_native_ctx, pv.path().c_str(),
+				   pv.value().c_str(), 0, NULL);
+		if (err != LY_SUCCESS) {
+			lyd_free_tree(input_tree);
+			return grpc::Status(grpc::StatusCode::INVALID_ARGUMENT,
+					    "Invalid input data");
+		}
+	}
+
+	// Validate input data.
+	err = lyd_validate_op(input_tree, NULL, LYD_TYPE_RPC_YANG, NULL);
+	if (err != LY_SUCCESS) {
+		lyd_free_tree(input_tree);
+		return grpc::Status(grpc::StatusCode::INVALID_ARGUMENT,
+				    "Invalid input data");
+	}
+
+	// Create output data tree.
+	err = lyd_new_path2(NULL, ly_native_ctx, xpath, NULL, 0,
+			    (LYD_ANYDATA_VALUETYPE)0, 0, NULL, &output_tree);
+	if (err != LY_SUCCESS) {
+		lyd_free_tree(input_tree);
+		return grpc::Status(grpc::StatusCode::INVALID_ARGUMENT,
+				    "Invalid data path");
 	}
 
 	// Execute callback registered for this XPath.
-	if (nb_callback_rpc(nb_node, xpath, input_list, output_list, errmsg,
-			    sizeof(errmsg))
-	    != NB_OK) {
+	if (nb_callback_rpc(nb_node, xpath, input_tree, output_tree, errmsg,
+			    sizeof(errmsg)) != NB_OK) {
 		flog_warn(EC_LIB_NB_CB_RPC, "%s: rpc callback failed: %s",
 			  __func__, xpath);
-		list_delete(&input_list);
-		list_delete(&output_list);
+		lyd_free_tree(input_tree);
+		lyd_free_tree(output_tree);
 
 		return grpc::Status(grpc::StatusCode::INTERNAL, "RPC failed");
 	}
 
 	// Process output parameters.
-	for (ALL_LIST_ELEMENTS_RO(output_list, node, data)) {
+	LY_LIST_FOR (lyd_child(output_tree), child) {
 		// Response: repeated PathValue output = 1;
 		frr::PathValue *pv = tag->response.add_output();
-		pv->set_path(data->xpath);
-		pv->set_value(data->value);
+		pv->set_path(lyd_path(child, LYD_PATH_STD, path, sizeof(path)));
+		pv->set_value(yang_dnode_get_string(child, NULL));
 	}
 
 	// Release memory.
-	list_delete(&input_list);
-	list_delete(&output_list);
+	lyd_free_tree(input_tree);
+	lyd_free_tree(output_tree);
 
 	return grpc::Status::OK;
 }
