@@ -99,6 +99,8 @@ static int nhrp_if_delete_hook(struct interface *ifp)
 		free(nifp->ipsec_fallback_profile);
 	if (nifp->source)
 		free(nifp->source);
+	if (nifp->auth_token)
+		zbuf_free(nifp->auth_token);
 
 	XFREE(MTYPE_NHRP_IF, ifp->info);
 	return 0;
@@ -267,6 +269,10 @@ static void nhrp_interface_update_address(struct interface *ifp, afi_t afi,
 	frr_each (if_connected, ifp->connected, c) {
 		if (PREFIX_FAMILY(c->address) != family)
 			continue;
+		/* On NHRPv6 interfaces a host prefix is required */
+		if (if_ad->configured && afi != AFI_IP
+		    && c->address->prefixlen != 8 * prefix_blen(c->address))
+			continue;
 		if (best == NULL) {
 			best = c;
 			continue;
@@ -287,14 +293,6 @@ static void nhrp_interface_update_address(struct interface *ifp, afi_t afi,
 			continue;
 	}
 
-	/* On NHRP interfaces a host prefix is required */
-	if (best && if_ad->configured
-	    && best->address->prefixlen != 8 * prefix_blen(best->address)) {
-		zlog_notice("%s: %pFX is not a host prefix", ifp->name,
-			    best->address);
-		best = NULL;
-	}
-
 	/* Update address if it changed */
 	if (best)
 		prefix2sockunion(best->address, &addr);
