@@ -186,6 +186,16 @@
 #define IPADDRESS   ASN_IPADDRESS
 #define STRING      ASN_OCTET_STR
 
+/* OSPFv3 MIB Interface State Values */
+#define OSPF6_SNMP_MIB_INTERFACE_DOWN          1
+#define OSPF6_SNMP_MIB_INTERFACE_LOOPBACK      2
+#define OSPF6_SNMP_MIB_INTERFACE_WAITING       3
+#define OSPF6_SNMP_MIB_INTERFACE_POINTTOPOINT  4
+#define OSPF6_SNMP_MIB_INTERFACE_DR            5
+#define OSPF6_SNMP_MIB_INTERFACE_BDR           6
+#define OSPF6_SNMP_MIB_INTERFACE_DROTHER       7
+#define OSPF6_SNMP_MIB_INTERFACE_UNKNOWN       8
+
 /* For return values e.g. SNMP_INTEGER macro */
 SNMP_LOCAL_VARIABLES
 
@@ -697,8 +707,8 @@ static uint8_t *ospfv3GeneralGroup(struct variable *v, oid *name,
 	case OSPFv3REFERENCEBANDWIDTH:
 		if (ospf6)
 			return SNMP_INTEGER(ospf6->ref_bandwidth);
-	/* Otherwise, like for "not implemented". */
-	/* fallthru */
+		/* Otherwise, like for "not implemented". */
+		return NULL;
 	case OSPFv3RESTARTSUPPORT:
 	case OSPFv3RESTARTINTERVAL:
 	case OSPFv3RESTARTSTRICTLSACHECKING:
@@ -748,8 +758,6 @@ static uint8_t *ospfv3AreaEntry(struct variable *v, oid *name, size_t *length,
 		area_id = htonl(name[v->namelen]);
 
 	inet_ntop(AF_INET, &area_id, a, sizeof(a));
-	zlog_debug("SNMP access by area: %s, exact=%d len=%d length=%lu", a,
-		   exact, len, (unsigned long)*length);
 
 	for (ALL_LIST_ELEMENTS_RO(ospf6->area_list, node, oa)) {
 		if (area == NULL) {
@@ -769,8 +777,6 @@ static uint8_t *ospfv3AreaEntry(struct variable *v, oid *name, size_t *length,
 	name[v->namelen] = ntohl(area->area_id);
 
 	inet_ntop(AF_INET, &area->area_id, a, sizeof(a));
-	zlog_debug("SNMP found area: %s, exact=%d len=%d length=%lu", a, exact,
-		   len, (unsigned long)*length);
 
 	switch (v->magic) {
 	case OSPFv3IMPORTASEXTERN:
@@ -1017,7 +1023,7 @@ static uint8_t *ospfv3WwLsdbEntry(struct variable *v, oid *name, size_t *length,
 	case OSPFv3WWLSDBCHECKSUM:
 		return SNMP_INTEGER(ntohs(lsa->header->checksum));
 	case OSPFv3WWLSDBADVERTISEMENT:
-		*var_len = ntohs(lsa->header->length);
+		*var_len = ospf6_lsa_size(lsa->header);
 		return (uint8_t *)lsa->header;
 	case OSPFv3WWLSDBTYPEKNOWN:
 		return SNMP_INTEGER(OSPF6_LSA_IS_KNOWN(lsa->header->type)
@@ -1027,6 +1033,36 @@ static uint8_t *ospfv3WwLsdbEntry(struct variable *v, oid *name, size_t *length,
 	return NULL;
 }
 
+/*
+ * Map OSPFv3 internal interface states to OSPFv3 MIB state values
+ *
+ * This function converts the internal interface state values used by OSPFv3
+ * into the corresponding values defined in the OSPFv3 Management Information
+ * Base (MIB). The MIB defines specific states for interfaces, which are
+ * used by SNMP to monitor and manage OSPFv3 networks.
+ */
+static uint8_t ospf6_snmp_interface_state(uint8_t state)
+{
+	switch (state) {
+	case OSPF6_INTERFACE_DOWN:
+		return OSPF6_SNMP_MIB_INTERFACE_DOWN;
+	case OSPF6_INTERFACE_LOOPBACK:
+		return OSPF6_SNMP_MIB_INTERFACE_LOOPBACK;
+	case OSPF6_INTERFACE_WAITING:
+		return OSPF6_SNMP_MIB_INTERFACE_WAITING;
+	case OSPF6_INTERFACE_POINTTOPOINT:
+		return OSPF6_SNMP_MIB_INTERFACE_POINTTOPOINT;
+	case OSPF6_INTERFACE_DR:
+		return OSPF6_SNMP_MIB_INTERFACE_DR;
+	case OSPF6_INTERFACE_BDR:
+		return OSPF6_SNMP_MIB_INTERFACE_BDR;
+	case OSPF6_INTERFACE_DROTHER:
+		return OSPF6_SNMP_MIB_INTERFACE_DROTHER;
+	default:
+		return OSPF6_SNMP_MIB_INTERFACE_UNKNOWN;
+	}
+}
+
 static uint8_t *ospfv3IfEntry(struct variable *v, oid *name, size_t *length,
 			      int exact, size_t *var_len,
 			      WriteMethod **write_method)
@@ -1126,6 +1162,8 @@ static uint8_t *ospfv3IfEntry(struct variable *v, oid *name, size_t *length,
 			return SNMP_INTEGER(1);
 		else if (oi->type == OSPF_IFTYPE_POINTOPOINT)
 			return SNMP_INTEGER(3);
+		else if (oi->type == OSPF_IFTYPE_POINTOMULTIPOINT)
+			return SNMP_INTEGER(5);
 		else
 			break; /* Unknown, don't put anything */
 	case OSPFv3IFADMINSTATUS:
@@ -1146,7 +1184,7 @@ static uint8_t *ospfv3IfEntry(struct variable *v, oid *name, size_t *length,
 		/* No support for NBMA */
 		break;
 	case OSPFv3IFSTATE:
-		return SNMP_INTEGER(oi->state);
+		return SNMP_INTEGER(ospf6_snmp_interface_state(oi->state));
 	case OSPFv3IFDESIGNATEDROUTER:
 		return SNMP_INTEGER(ntohl(oi->drouter));
 	case OSPFv3IFBACKUPDESIGNATEDROUTER:
@@ -1367,6 +1405,7 @@ static int ospf6TrapIfStateChange(struct ospf6_interface *oi, int next_state,
 
 	/* Terminal state or regression */
 	if ((next_state != OSPF6_INTERFACE_POINTTOPOINT)
+	    && (next_state != OSPF6_INTERFACE_POINTTOMULTIPOINT)
 	    && (next_state != OSPF6_INTERFACE_DROTHER)
 	    && (next_state != OSPF6_INTERFACE_BDR)
 	    && (next_state != OSPF6_INTERFACE_DR) && (next_state >= prev_state))
@@ -1383,9 +1422,9 @@ static int ospf6TrapIfStateChange(struct ospf6_interface *oi, int next_state,
 }
 
 /* Register OSPFv3-MIB. */
-static int ospf6_snmp_init(struct event_loop *master)
+static int ospf6_snmp_init(struct event_loop *mstr)
 {
-	smux_init(master);
+	smux_init(mstr);
 	REGISTER_MIB("OSPFv3MIB", ospfv3_variables, variable, ospfv3_oid);
 	return 0;
 }
