@@ -1,20 +1,7 @@
+// SPDX-License-Identifier: GPL-2.0-or-later
 /*
  * PIM for Quagga
  * Copyright (C) 2008  Everton da Silva Marques
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; see the file COPYING; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #ifndef PIM_NEIGHBOR_H
@@ -40,10 +27,10 @@ struct pim_neighbor {
 	uint32_t dr_priority;
 	uint32_t generation_id;
 	struct list *prefix_list; /* list of struct prefix */
-	struct thread *t_expire_timer;
+	struct event *t_expire_timer;
 	struct interface *interface;
 
-	struct thread *jp_timer;
+	struct event *jp_timer;
 	struct list *upstream_jp_agg;
 	struct bfd_session_params *bfd_session;
 };
