@@ -8,12 +8,21 @@
 #define PIM_UPSTREAM_H
 
 #include <zebra.h>
-#include <prefix.h>
-#include "plist.h"
 
-#include "pim_rpf.h"
-#include "pim_str.h"
-#include "pim_ifchannel.h"
+#include <stddef.h>
+#include <stdbool.h>
+#include <stdint.h>
+
+#include "lib/typesafe.h"
+
+#include "pimd/pim_addr.h"
+#include "pimd/pim_rpf.h"
+#include "pimd/pim_str.h"
+
+struct interface;
+struct pim_ifchannel;
+struct pim_instance;
+struct prefix_list;
 
 #define PIM_UPSTREAM_FLAG_MASK_DR_JOIN_DESIRED         (1 << 0)
 #define PIM_UPSTREAM_FLAG_MASK_DR_JOIN_DESIRED_UPDATED (1 << 1)
@@ -81,9 +90,15 @@
  */
 #define PIM_UPSTREAM_FLAG_MASK_MLAG_INTERFACE          (1 << 21)
 
+#define PIM_UPSTREAM_DM_FLAG_MASK_INTERFACE            (1 << 22)
+
+#define PIM_UPSTREAM_DM_FLAG_MASK_PRUNE                (1 << 23)
+
 
 #define PIM_UPSTREAM_FLAG_ALL 0xFFFFFFFF
 
+#define PIM_UPSTREAM_DM_TEST_PRUNE(flags) ((flags) & PIM_UPSTREAM_DM_FLAG_MASK_PRUNE)
+#define PIM_UPSTREAM_DM_TEST_INTERFACE(flags) ((flags) & PIM_UPSTREAM_DM_FLAG_MASK_INTERFACE)
 #define PIM_UPSTREAM_FLAG_TEST_DR_JOIN_DESIRED(flags) ((flags) & PIM_UPSTREAM_FLAG_MASK_DR_JOIN_DESIRED)
 #define PIM_UPSTREAM_FLAG_TEST_DR_JOIN_DESIRED_UPDATED(flags) ((flags) & PIM_UPSTREAM_FLAG_MASK_DR_JOIN_DESIRED_UPDATED)
 #define PIM_UPSTREAM_FLAG_TEST_FHR(flags) ((flags) & PIM_UPSTREAM_FLAG_MASK_FHR)
@@ -109,6 +124,8 @@
 #define PIM_UPSTREAM_FLAG_TEST_CAN_BE_LHR(flags) ((flags) & (PIM_UPSTREAM_FLAG_MASK_SRC_IGMP | PIM_UPSTREAM_FLAG_MASK_SRC_VXLAN_TERM))
 #define PIM_UPSTREAM_FLAG_TEST_MLAG_INTERFACE(flags) ((flags)&PIM_UPSTREAM_FLAG_MASK_MLAG_INTERFACE)
 
+#define PIM_UPSTREAM_DM_SET_PRUNE(flags) ((flags) |= PIM_UPSTREAM_DM_FLAG_MASK_PRUNE)
+#define PIM_UPSTREAM_DM_SET_INTERFACE(flags) ((flags) |= PIM_UPSTREAM_DM_FLAG_MASK_INTERFACE)
 #define PIM_UPSTREAM_FLAG_SET_DR_JOIN_DESIRED(flags) ((flags) |= PIM_UPSTREAM_FLAG_MASK_DR_JOIN_DESIRED)
 #define PIM_UPSTREAM_FLAG_SET_DR_JOIN_DESIRED_UPDATED(flags) ((flags) |= PIM_UPSTREAM_FLAG_MASK_DR_JOIN_DESIRED_UPDATED)
 #define PIM_UPSTREAM_FLAG_SET_FHR(flags) ((flags) |= PIM_UPSTREAM_FLAG_MASK_FHR)
@@ -131,6 +148,8 @@
 #define PIM_UPSTREAM_FLAG_SET_USE_RPT(flags) ((flags) |= PIM_UPSTREAM_FLAG_MASK_USE_RPT)
 #define PIM_UPSTREAM_FLAG_SET_MLAG_INTERFACE(flags) ((flags) |= PIM_UPSTREAM_FLAG_MASK_MLAG_INTERFACE)
 
+#define PIM_UPSTREAM_DM_UNSET_PRUNE(flags) ((flags) &= ~PIM_UPSTREAM_DM_FLAG_MASK_PRUNE)
+
 #define PIM_UPSTREAM_FLAG_UNSET_DR_JOIN_DESIRED(flags) ((flags) &= ~PIM_UPSTREAM_FLAG_MASK_DR_JOIN_DESIRED)
 #define PIM_UPSTREAM_FLAG_UNSET_DR_JOIN_DESIRED_UPDATED(flags) ((flags) &= ~PIM_UPSTREAM_FLAG_MASK_DR_JOIN_DESIRED_UPDATED)
 #define PIM_UPSTREAM_FLAG_UNSET_FHR(flags) ((flags) &= ~PIM_UPSTREAM_FLAG_MASK_FHR)
@@ -238,6 +257,10 @@ struct pim_upstream {
 	struct pim_up_mlag mlag;
 
 	struct event *t_join_timer;
+	struct event *t_prune_timer;
+	struct event *t_staterefresh_timer;
+	struct event *t_graft_timer;
+	struct event *t_prune_limit_timer;
 
 	/*
 	 * RST(S,G)
@@ -251,6 +274,7 @@ struct pim_upstream {
 	 */
 	struct event *t_ka_timer;
 #define PIM_KEEPALIVE_PERIOD  (210)
+#define PIM_STATEREFRESH_PERIOD (60)
 #define PIM_RP_KEEPALIVE_PERIOD                                                \
 	(3 * router->register_suppress_time + router->register_probe_time)
 
@@ -331,8 +355,7 @@ int pim_upstream_is_sg_rpt(struct pim_upstream *up);
 void pim_upstream_set_sptbit(struct pim_upstream *up,
 			     struct interface *incoming);
 
-void pim_upstream_start_register_stop_timer(struct pim_upstream *up,
-					    int null_register);
+void pim_upstream_start_register_probe_timer(struct pim_upstream *up);
 
 void pim_upstream_send_join(struct pim_upstream *up);
 
@@ -351,12 +374,16 @@ int pim_upstream_inherited_olist(struct pim_instance *pim,
 int pim_upstream_empty_inherited_olist(struct pim_upstream *up);
 
 void pim_upstream_find_new_rpf(struct pim_instance *pim);
-void pim_upstream_msdp_reg_timer_start(struct pim_upstream *up);
 
 void pim_upstream_init(struct pim_instance *pim);
 void pim_upstream_terminate(struct pim_instance *pim);
 
 void join_timer_start(struct pim_upstream *up);
+void staterefresh_timer_start(struct pim_upstream *up);
+void graft_timer_start(struct pim_upstream *up);
+void prune_timer_start(struct pim_upstream *up);
+void prune_limit_timer_start(struct pim_upstream *up);
+bool should_limit_prune(struct pim_upstream *up);
 int pim_upstream_compare(const struct pim_upstream *up1,
 			 const struct pim_upstream *up2);
 DECLARE_RBTREE_UNIQ(rb_pim_upstream, struct pim_upstream, upstream_rb,
@@ -383,4 +410,7 @@ uint32_t pim_up_mlag_local_cost(struct pim_upstream *up);
 uint32_t pim_up_mlag_peer_cost(struct pim_upstream *up);
 void pim_upstream_reeval_use_rpt(struct pim_instance *pim);
 int pim_upstream_could_register(struct pim_upstream *up);
+bool pim_sg_is_reevaluate_oil_req(struct pim_instance *pim,
+				  struct pim_upstream *up);
+bool pim_upstream_up_connected(struct pim_upstream *up);
 #endif /* PIM_UPSTREAM_H */
