@@ -12,7 +12,6 @@ via BGP dynamic capability.
 """
 
 import os
-import re
 import sys
 import json
 import pytest
@@ -25,7 +24,7 @@ sys.path.append(os.path.join(CWD, "../"))
 
 # pylint: disable=C0413
 from lib import topotest
-from lib.topogen import Topogen, TopoRouter, get_topogen
+from lib.topogen import Topogen, get_topogen
 from lib.common_config import step
 
 
@@ -223,7 +222,7 @@ def test_bgp_dynamic_capability_graceful_restart():
                 "bgpState": "Established",
                 "neighborCapabilities": {
                     "dynamic": "advertisedAndReceived",
-                    "gracefulRestartCapability": "received",
+                    "gracefulRestart": "received",
                     "longLivedGracefulRestart": "received",
                 },
                 "messageStats": {
