@@ -134,26 +134,13 @@ def test_vrf_route_leak_donna():
                 ],
             },
         ],
-        "172.16.101.0/24": [
-            {
-                "protocol": "bgp",
-                "selected": None,
-                "nexthops": [
-                    {
-                        "fib": None,
-                        "interfaceName": "unknown",
-                        "vrf": "Unknown",
-                        "active": None,
-                    },
-                ],
-            },
-        ],
+        "172.16.101.0/24": None,
     }
 
     test_func = partial(
         topotest.router_json_cmp, r1, "show ip route vrf DONNA json", expect
     )
-    result, diff = topotest.run_and_expect(test_func, None, count=10, wait=0.5)
+    result, diff = topotest.run_and_expect(test_func, None, count=15, wait=1)
     assert result, "BGP VRF DONNA check failed:\n{}".format(diff)
 
 
@@ -206,26 +193,13 @@ def test_vrf_route_leak_eva():
                 "protocol": "connected",
             }
         ],
-        "172.16.101.0/24": [
-            {
-                "protocol": "bgp",
-                "selected": None,
-                "nexthops": [
-                    {
-                        "fib": None,
-                        "interfaceName": "unknown",
-                        "vrf": "Unknown",
-                        "active": None,
-                    },
-                ],
-            },
-        ],
+        "172.16.101.0/24": None,
     }
 
     test_func = partial(
         topotest.router_json_cmp, r1, "show ip route vrf EVA json", expect
     )
-    result, diff = topotest.run_and_expect(test_func, None, count=10, wait=0.5)
+    result, diff = topotest.run_and_expect(test_func, None, count=15, wait=1)
     assert result, "BGP VRF EVA check failed:\n{}".format(diff)
 
 
@@ -276,7 +250,7 @@ def test_vrf_route_leak_default():
     }
 
     test_func = partial(topotest.router_json_cmp, r1, "show ip route json", expect)
-    result, diff = topotest.run_and_expect(test_func, None, count=10, wait=0.5)
+    result, diff = topotest.run_and_expect(test_func, None, count=15, wait=1)
     assert result, "BGP VRF default check failed:\n{}".format(diff)
 
 
@@ -313,40 +287,14 @@ interface EVA
 
     # Test DONNA VRF.
     expect = {
-        "10.0.1.0/24": [
-            {
-                "protocol": "bgp",
-                "selected": None,
-                "nexthops": [
-                    {
-                        "fib": None,
-                        "interfaceName": "EVA",
-                        "vrf": "EVA",
-                        "active": None,
-                    },
-                ],
-            },
-        ],
-        "10.0.3.0/24": [
-            {
-                "protocol": "bgp",
-                "selected": None,
-                "nexthops": [
-                    {
-                        "fib": None,
-                        "interfaceName": "EVA",
-                        "vrf": "EVA",
-                        "active": None,
-                    },
-                ],
-            },
-        ],
+        "10.0.1.0/24": None,
+        "10.0.3.0/24": None,
     }
 
     test_func = partial(
         topotest.router_json_cmp, r1, "show ip route vrf DONNA json", expect
     )
-    result, diff = topotest.run_and_expect(test_func, None, count=10, wait=0.5)
+    result, diff = topotest.run_and_expect(test_func, None, count=15, wait=1)
     assert result, "BGP VRF DONNA check failed:\n{}".format(diff)
 
     """
@@ -361,7 +309,7 @@ interface EVA
         return topotest.json_cmp(vrf_table, expect)
 
     test_func = partial(check_vrf_table, r1, "DONNA", expect)
-    result, diff = topotest.run_and_expect(test_func, None, count=10, wait=0.5)
+    result, diff = topotest.run_and_expect(test_func, None, count=15, wait=1)
     assert result, "BGP VRF DONNA check failed:\n{}".format(diff)
 
 
@@ -416,7 +364,7 @@ interface EVA
     test_func = partial(
         topotest.router_json_cmp, r1, "show ip route vrf DONNA json", expect
     )
-    result, diff = topotest.run_and_expect(test_func, None, count=10, wait=0.5)
+    result, diff = topotest.run_and_expect(test_func, None, count=15, wait=1)
     assert result, "BGP VRF DONNA check failed:\n{}".format(diff)
 
 
@@ -432,26 +380,13 @@ def test_vrf_route_leak_donna_add_vrf_zita():
 
     # Test DONNA VRF.
     expect = {
-        "172.16.101.0/24": [
-            {
-                "protocol": "bgp",
-                "selected": None,
-                "nexthops": [
-                    {
-                        "fib": None,
-                        "interfaceName": "ZITA",
-                        "vrf": "ZITA",
-                        "active": None,
-                    },
-                ],
-            },
-        ],
+        "172.16.101.0/24": None,
     }
 
     test_func = partial(
         topotest.router_json_cmp, r1, "show ip route vrf DONNA json", expect
     )
-    result, diff = topotest.run_and_expect(test_func, None, count=10, wait=0.5)
+    result, diff = topotest.run_and_expect(test_func, None, count=15, wait=1)
     assert result, "BGP VRF DONNA check failed:\n{}".format(diff)
 
 
@@ -492,7 +427,7 @@ interface ZITA
     test_func = partial(
         topotest.router_json_cmp, r1, "show ip route vrf DONNA json", expect
     )
-    result, diff = topotest.run_and_expect(test_func, None, count=10, wait=0.5)
+    result, diff = topotest.run_and_expect(test_func, None, count=15, wait=1)
     assert result, "BGP VRF DONNA check failed:\n{}".format(diff)
 
 
@@ -514,7 +449,7 @@ def test_vrf_route_leak_donna_delete_vrf_zita():
     test_func = partial(
         topotest.router_json_cmp, r1, "show ip route vrf DONNA json", expect
     )
-    result, diff = topotest.run_and_expect(test_func, None, count=10, wait=0.5)
+    result, diff = topotest.run_and_expect(test_func, None, count=15, wait=1)
     assert result, "BGP VRF DONNA check failed:\n{}".format(diff)
 
 
