@@ -16,7 +16,6 @@ import signal
 import subprocess
 import sys
 import time
-from datetime import datetime, timedelta
 from functools import partial
 
 import pytest
@@ -35,8 +34,7 @@ from lib.topotest import interface_set_status, json_cmp
 # pylint: disable=C0413
 # Import topogen and topotest helpers
 from lib import topotest
-from lib.topogen import Topogen, TopoRouter, get_topogen
-from lib.topolog import logger
+from lib.topogen import Topogen, TopoRouter
 
 pytestmark = [pytest.mark.ospfd]
 
@@ -220,10 +218,12 @@ def _test_router_id(tgen, testbin):
 
         step("router id: check for modified router id")
         r1.vtysh_multicmd("conf t\nrouter ospf\nospf router-id 1.1.1.1")
+        r1.vtysh_multicmd("clear ip ospf process")
         _wait_output(p, "SUCCESS: {}".format(waitlist[1]))
 
         step("router id: check for restored router id")
         r1.vtysh_multicmd("conf t\nrouter ospf\nospf router-id 1.0.0.0")
+        r1.vtysh_multicmd("clear ip ospf process")
         _wait_output(p, "SUCCESS: {}".format(waitlist[2]))
     except Exception as error:
         logging.error("ERROR: %s", error)
@@ -1681,6 +1681,32 @@ def _test_opaque_link_local_lsa_crash(tgen, apibin):
         step("Bring r1-eth0 back up and verify there is no crash")
         r1.vtysh_multicmd("conf t\ninterface r1-eth0\nno shut")
 
+        # Verify that the neighbor has come back up
+        step("Verify neighbor has come back up")
+        expected_neighbor = {
+            "neighbors": {
+                "2.0.0.0": [
+                    {
+                        "nbrState": "Full/DR",
+                        "nbrPriority": 1,
+                        "converged": "Full",
+                        "role": "DR",
+                        "ifaceAddress": "10.0.1.2",
+                        "ifaceName": "r1-eth0:10.0.1.1",
+                    }
+                ]
+            }
+        }
+        test_func = partial(
+            topotest.router_json_cmp,
+            r1,
+            "show ip ospf neigh json",
+            expected_neighbor,
+        )
+        _, result = topotest.run_and_expect(test_func, None, count=60, wait=1)
+        assertmsg = "r1 OSPF neighbor has not come back up after interface restart"
+        assert result is None, assertmsg
+
         step("Add another link-local opaque LSA for r1-eth0")
         pread = r1.popen([apibin, "-v", "add,9,10.0.1.1,230,1,feedaceecafebeef"])
 
