@@ -65,19 +65,19 @@ extern struct event_loop *master;
 #define VTYSH_AFFMAP VTYSH_ISISD | VTYSH_MGMTD
 #define VTYSH_RMAP_CONFIG                                                      \
 	VTYSH_OSPFD | VTYSH_OSPF6D | VTYSH_BGPD | VTYSH_ISISD |  \
-		VTYSH_PIMD | VTYSH_EIGRPD | VTYSH_FABRICD | VTYSH_MGMTD
+		VTYSH_PIMD | VTYSH_PIM6D | VTYSH_EIGRPD | VTYSH_FABRICD | VTYSH_MGMTD
 #define VTYSH_RMAP_SHOW                                                        \
 	VTYSH_ZEBRA | VTYSH_RIPD | VTYSH_RIPNGD | VTYSH_OSPFD | VTYSH_OSPF6D | \
-		VTYSH_BGPD | VTYSH_ISISD | VTYSH_PIMD | VTYSH_EIGRPD |         \
-		VTYSH_FABRICD
+		VTYSH_BGPD | VTYSH_ISISD | VTYSH_PIMD | VTYSH_PIM6D |          \
+		VTYSH_EIGRPD | VTYSH_FABRICD
 #define VTYSH_ACCESS_LIST_SHOW                                                 \
 	VTYSH_ZEBRA | VTYSH_RIPD | VTYSH_RIPNGD | VTYSH_OSPFD | VTYSH_OSPF6D | \
 		VTYSH_BGPD | VTYSH_ISISD | VTYSH_PIMD | VTYSH_EIGRPD |         \
 		VTYSH_FABRICD
 #define VTYSH_PREFIX_LIST_SHOW                                                 \
 	VTYSH_ZEBRA | VTYSH_RIPD | VTYSH_RIPNGD | VTYSH_OSPFD | VTYSH_OSPF6D | \
-		VTYSH_BGPD | VTYSH_ISISD | VTYSH_PIMD | VTYSH_EIGRPD |         \
-		VTYSH_FABRICD
+		VTYSH_BGPD | VTYSH_ISISD | VTYSH_PIMD | VTYSH_PIM6D |          \
+		VTYSH_EIGRPD | VTYSH_FABRICD
 #define VTYSH_INTERFACE_SUBSET                                                 \
 	VTYSH_OSPFD | VTYSH_OSPF6D | \
 		VTYSH_ISISD | VTYSH_PIMD | VTYSH_PIM6D | VTYSH_NHRPD |         \
@@ -94,6 +94,8 @@ extern struct event_loop *master;
 	VTYSH_RIPD | VTYSH_RIPNGD | VTYSH_STATICD | VTYSH_ZEBRA
 #define VTYSH_MGMT_FRONTEND VTYSH_MGMTD
 
+#define VTYSH_NON_MGMTD ((VTYSH_ALL) & ~(VTYSH_MGMT_BACKEND))
+
 enum vtysh_write_integrated {
 	WRITE_INTEGRATED_UNSPECIFIED,
 	WRITE_INTEGRATED_NO,
@@ -149,10 +151,15 @@ void vtysh_config_init(void);
 void suid_on(void);
 void suid_off(void);
 
+/* Handle exec-timeout config changes */
+#define VTYSH_EXEC_TIMEOUT_MAX 1000 * 1000
+void vtysh_exec_timeout_config(uint32_t tsecs);
+uint32_t vtysh_get_exec_timeout(void);
+
 /* Child process execution flag. */
 extern int execute_flag;
 
-extern struct vty *vty;
+extern struct vty *gvty;
 
 extern int user_mode;
 
