@@ -315,11 +315,20 @@ void vtysh_config_parse_line(void *arg, const char *line)
 			} else if (!strncmp(line, " ip mroute",
 					    strlen(" ip mroute"))) {
 				config_add_line_uniq_end(config->line, line);
+			} else if ((strncmp(line, " rpki", strlen(" rpki")) ==
+				    0) &&
+				   config->index == VRF_NODE) {
+				config_add_line(config->line, line);
+				config->index = RPKI_VRF_NODE;
 			} else if (config->index == RMAP_NODE ||
 				   config->index == INTERFACE_NODE ||
 				   config->index == VTY_NODE)
 				config_add_line_uniq(config->line, line);
-			else if (config->index == NH_GROUP_NODE) {
+			else if (config->index == RPKI_VRF_NODE &&
+				 strncmp(line, "  exit", strlen("  exit")) == 0) {
+				config_add_line(config->line, line);
+				config->index = VRF_NODE;
+			} else if (config->index == NH_GROUP_NODE) {
 				if (strncmp(line, " resilient",
 					    strlen(" resilient")) == 0)
 					config_add_line_head(config->line,
@@ -435,19 +444,8 @@ void vtysh_config_parse_line(void *arg, const char *line)
 			config = config_get(KEYCHAIN_NODE, line);
 		else if (strncmp(line, "line", strlen("line")) == 0)
 			config = config_get(VTY_NODE, line);
-		else if ((strncmp(line, "ipv6 forwarding",
-				  strlen("ipv6 forwarding"))
-			  == 0)
-			 || (strncmp(line, "ip forwarding",
-				     strlen("ip forwarding"))
-			     == 0))
-			config = config_get(FORWARDING_NODE, line);
 		else if (strncmp(line, "debug vrf", strlen("debug vrf")) == 0)
 			config = config_get(VRF_DEBUG_NODE, line);
-		else if (strncmp(line, "debug northbound",
-				 strlen("debug northbound"))
-			 == 0)
-			config = config_get(NORTHBOUND_DEBUG_NODE, line);
 		else if (strncmp(line, "debug route-map",
 				 strlen("debug route-map"))
 			 == 0)
@@ -482,6 +480,11 @@ void vtysh_config_parse_line(void *arg, const char *line)
 			config = config_get(BFD_NODE, line);
 		else if (strncmp(line, "rpki", strlen("rpki")) == 0)
 			config = config_get(RPKI_NODE, line);
+		else if (strncmp(line, "router pim", strlen("router pim")) == 0)
+			config = config_get(PIM_NODE, line);
+		else if (strncmp(line, "router pim6", strlen("router pim6")) ==
+			 0)
+			config = config_get(PIM6_NODE, line);
 		else {
 			if (strncmp(line, "log", strlen("log")) == 0 ||
 			    strncmp(line, "hostname", strlen("hostname")) == 0 ||
@@ -510,16 +513,12 @@ void vtysh_config_parse_line(void *arg, const char *line)
 
 /* Macro to check delimiter is needed between each configuration line
  * or not. */
-#define NO_DELIMITER(I)                                                        \
-	((I) == AFFMAP_NODE || (I) == ACCESS_NODE || (I) == PREFIX_NODE ||     \
-	 (I) == IP_NODE || (I) == AS_LIST_NODE ||                              \
-	 (I) == COMMUNITY_LIST_NODE || (I) == COMMUNITY_ALIAS_NODE ||          \
-	 (I) == ACCESS_IPV6_NODE || (I) == ACCESS_MAC_NODE ||                  \
-	 (I) == PREFIX_IPV6_NODE || (I) == FORWARDING_NODE ||                  \
-	 (I) == DEBUG_NODE || (I) == AAA_NODE || (I) == VRF_DEBUG_NODE ||      \
-	 (I) == NORTHBOUND_DEBUG_NODE || (I) == RMAP_DEBUG_NODE ||             \
-	 (I) == RESOLVER_DEBUG_NODE || (I) == MPLS_NODE ||                     \
-	 (I) == KEYCHAIN_KEY_NODE)
+#define NO_DELIMITER(I)                                                                             \
+	((I) == AFFMAP_NODE || (I) == ACCESS_NODE || (I) == PREFIX_NODE || (I) == IP_NODE ||        \
+	 (I) == AS_LIST_NODE || (I) == COMMUNITY_LIST_NODE || (I) == COMMUNITY_ALIAS_NODE ||        \
+	 (I) == ACCESS_IPV6_NODE || (I) == ACCESS_MAC_NODE || (I) == PREFIX_IPV6_NODE ||            \
+	 (I) == DEBUG_NODE || (I) == AAA_NODE || (I) == VRF_DEBUG_NODE || (I) == RMAP_DEBUG_NODE || \
+	 (I) == RESOLVER_DEBUG_NODE || (I) == MPLS_NODE || (I) == KEYCHAIN_KEY_NODE)
 
 static void configvec_dump(vector vec, bool nested)
 {
@@ -549,19 +548,19 @@ static void configvec_dump(vector vec, bool nested)
 					continue;
 				}
 
-				vty_out(vty, "%s\n", config->name);
+				vty_out(gvty, "%s\n", config->name);
 
 				for (ALL_LIST_ELEMENTS(config->line, mnode,
 						       mnnode, line))
-					vty_out(vty, "%s\n", line);
+					vty_out(gvty, "%s\n", line);
 
 				configvec_dump(config->nested, true);
 
 				if (config->exit)
-					vty_out(vty, "%s\n", config->exit);
+					vty_out(gvty, "%s\n", config->exit);
 
 				if (!NO_DELIMITER(i))
-					vty_out(vty, "!\n");
+					vty_out(gvty, "!\n");
 
 				config_del(config);
 			}
@@ -570,7 +569,7 @@ static void configvec_dump(vector vec, bool nested)
 			XFREE(MTYPE_VTYSH_CONFIG, configuration);
 			vector_slot(vec, i) = NULL;
 			if (!nested && NO_DELIMITER(i))
-				vty_out(vty, "!\n");
+				vty_out(gvty, "!\n");
 		}
 }
 
@@ -580,11 +579,11 @@ void vtysh_config_dump(void)
 	char *line;
 
 	for (ALL_LIST_ELEMENTS(config_top, node, nnode, line))
-		vty_out(vty, "%s\n", line);
+		vty_out(gvty, "%s\n", line);
 
 	list_delete_all_node(config_top);
 
-	vty_out(vty, "!\n");
+	vty_out(gvty, "!\n");
 
 	configvec_dump(configvec, false);
 }
@@ -592,34 +591,43 @@ void vtysh_config_dump(void)
 /* Read up configuration file from file_name. */
 static int vtysh_read_file(FILE *confp, bool dry_run)
 {
-	struct vty *vty;
-	int ret;
+	struct vty *lvty;
+	int ret, saved_ret = CMD_SUCCESS;
 
-	vty = vty_new();
-	vty->wfd = STDERR_FILENO;
-	vty->type = VTY_TERM;
-	vty->node = CONFIG_NODE;
+	lvty = vty_new();
+	lvty->wfd = STDERR_FILENO;
+	lvty->type = VTY_TERM;
+	lvty->node = CONFIG_NODE;
 
 	vtysh_execute_no_pager("enable");
-	vtysh_execute_no_pager("conf term file-lock");
-	vty->vtysh_file_locked = true;
+	/*
+	 * When reading the config, we need to wait until the lock is acquired.
+	 * If we ignore the failure and continue without the lock, the config
+	 * will be fully ignored.
+	 */
+	while (vtysh_execute_no_pager("conf term file-lock") == CMD_WARNING_CONFIG_FAILED)
+		usleep(100000);
 
 	if (!dry_run)
 		vtysh_execute_no_pager("XFRR_start_configuration");
 
 	/* Execute configuration file. */
-	ret = vtysh_config_from_file(vty, confp);
+	ret = vtysh_config_from_file(lvty, confp);
+	if (ret != CMD_SUCCESS)
+		saved_ret = ret;
 
-	if (!dry_run)
-		vtysh_execute_no_pager("XFRR_end_configuration");
+	if (!dry_run) {
+		ret = vtysh_execute_no_pager("XFRR_end_configuration");
+		if (ret != CMD_SUCCESS)
+			saved_ret = ret;
+	}
 
 	vtysh_execute_no_pager("end");
-	vty->vtysh_file_locked = false;
 	vtysh_execute_no_pager("disable");
 
-	vty_close(vty);
+	vty_close(lvty);
 
-	return (ret);
+	return (saved_ret);
 }
 
 /*
@@ -754,6 +762,7 @@ void vtysh_config_write(void)
 {
 	const char *name;
 	char line[512];
+	uint32_t ival;
 
 	name = cmd_hostname_get();
 	if (name && name[0] != '\0') {
@@ -774,6 +783,12 @@ void vtysh_config_write(void)
 		vtysh_config_parse_line(NULL,
 					"service integrated-vtysh-config");
 
+	ival = vtysh_get_exec_timeout();
+	if (ival > 0) {
+		snprintf(line, sizeof(line), "exec-timeout %u", ival);
+		vtysh_config_parse_line(NULL, line);
+	}
+
 	user_config_write();
 }
 
