@@ -140,9 +140,9 @@ void zebra_ptm_finish(void)
 		free(ptm_cb.in_data);
 
 	/* Cancel events. */
-	EVENT_OFF(ptm_cb.t_read);
-	EVENT_OFF(ptm_cb.t_write);
-	EVENT_OFF(ptm_cb.t_timer);
+	event_cancel(&ptm_cb.t_read);
+	event_cancel(&ptm_cb.t_write);
+	event_cancel(&ptm_cb.t_timer);
 
 	if (ptm_cb.wb)
 		buffer_free(ptm_cb.wb);
@@ -196,7 +196,7 @@ static int zebra_ptm_send_message(char *data, int size)
 				ptm_cb.reconnect_time, &ptm_cb.t_timer);
 		return -1;
 	case BUFFER_EMPTY:
-		EVENT_OFF(ptm_cb.t_write);
+		event_cancel(&ptm_cb.t_write);
 		break;
 	case BUFFER_PENDING:
 		event_add_write(zrouter.master, zebra_ptm_flush_messages, NULL,
@@ -1224,7 +1224,6 @@ static void pp_free_all(void)
  */
 static void zebra_ptm_send_bfdd(struct stream *msg)
 {
-	struct listnode *node;
 	struct zserv *client;
 	struct stream *msgc;
 
@@ -1232,7 +1231,7 @@ static void zebra_ptm_send_bfdd(struct stream *msg)
 	msgc = stream_dup(msg);
 
 	/* Send message to all running BFDd daemons. */
-	for (ALL_LIST_ELEMENTS_RO(zrouter.client_list, node, client)) {
+	frr_each (zserv_client_list, &zrouter.client_list, client) {
 		if (client->proto != ZEBRA_ROUTE_BFD)
 			continue;
 
@@ -1248,7 +1247,6 @@ static void zebra_ptm_send_bfdd(struct stream *msg)
 
 static void zebra_ptm_send_clients(struct stream *msg)
 {
-	struct listnode *node;
 	struct zserv *client;
 	struct stream *msgc;
 
@@ -1256,7 +1254,7 @@ static void zebra_ptm_send_clients(struct stream *msg)
 	msgc = stream_dup(msg);
 
 	/* Send message to all running client daemons. */
-	for (ALL_LIST_ELEMENTS_RO(zrouter.client_list, node, client)) {
+	frr_each (zserv_client_list, &zrouter.client_list, client) {
 		if (!IS_BFD_ENABLED_PROTOCOL(client->proto))
 			continue;
 
